local _hts_PlayerStandard_init = PlayerStandard.init

function PlayerStandard:init(...)
	_hts_PlayerStandard_init(self, ...)
	self._melee_hts_t = 0
end

local AUTORELOAD_DELAY = 0.25
local AUTORELOAD_EXPIRATION = math.max(AUTORELOAD_DELAY, 0)
 
local _end_action_charging_weapon_original = PlayerStandard._end_action_charging_weapon
local _check_action_reload_original = PlayerStandard._check_action_reload
local inventory_clbk_listener_original = PlayerStandard.inventory_clbk_listener
 
function PlayerStandard:_end_action_charging_weapon(t, ...)
    if self._equipped_unit:base().clip_empty and self._equipped_unit:base():clip_empty() then
        self._auto_reload_t = t
    end
    return _end_action_charging_weapon_original(self, t, ...)
end
 
function PlayerStandard:_check_action_reload(t, input, ...)
    input.btn_reload_press = input.btn_reload_press or (self._auto_reload_t and self._auto_reload_t + AUTORELOAD_DELAY < t)
    local new_action = _check_action_reload_original(self, t, input, ...)
    if new_action or (self._auto_reload_t and self._auto_reload_t + AUTORELOAD_EXPIRATION < t) then
        self._auto_reload_t = nil
    end
    return new_action
end
 
function PlayerStandard:inventory_clbk_listener(...)
    self._auto_reload_t = nil
    return inventory_clbk_listener_original(self, ...)
end

local melee_vars = {
    "player_melee",
    "player_melee_var2"
}
function PlayerStandard:_do_melee_damage(t, bayonet_melee, melee_hit_ray)
    local melee_entry = managers.blackmarket:equipped_melee_weapon()
    local instant_hit = tweak_data.blackmarket.melee_weapons[melee_entry].instant
    local melee_damage_delay = tweak_data.blackmarket.melee_weapons[melee_entry].melee_damage_delay or 0
    local charge_lerp_value = instant_hit and 0 or self:_get_melee_charge_lerp_value(t, melee_damage_delay)
    self._ext_camera:play_shaker(melee_vars[math.random(#melee_vars)], math.max(0.3, charge_lerp_value))
    local sphere_cast_radius = 20
    local col_ray
    if melee_hit_ray then
        col_ray = melee_hit_ray ~= true and melee_hit_ray or nil
    else
        col_ray = self:_calc_melee_hit_ray(t, sphere_cast_radius)
    end
    if col_ray and alive(col_ray.unit) then
        local damage, damage_effect = managers.blackmarket:equipped_melee_weapon_damage_info(charge_lerp_value)
        local damage_effect_mul = math.max(managers.player:upgrade_value("player", "melee_knockdown_mul", 1), managers.player:upgrade_value(self._equipped_unit:base():weapon_tweak_data().categories and self._equipped_unit:base():weapon_tweak_data().categories[1], "melee_knockdown_mul", 1))
        damage = damage * managers.player:get_melee_dmg_multiplier()
        damage_effect = damage_effect * damage_effect_mul
        col_ray.sphere_cast_radius = sphere_cast_radius
        local hit_unit = col_ray.unit
        if hit_unit:character_damage() then
            if bayonet_melee then
                self._unit:sound():play("fairbairn_hit_body", nil, false)
            else
                local hit_sfx = "hit_body"
                if hit_unit:character_damage() and hit_unit:character_damage().melee_hit_sfx then
                    hit_sfx = hit_unit:character_damage():melee_hit_sfx()
                end
                self:_play_melee_sound(melee_entry, hit_sfx, self._melee_attack_var)
            end
            if not hit_unit:character_damage()._no_blood then
                managers.game_play_central:play_impact_flesh({col_ray = col_ray})
                managers.game_play_central:play_impact_sound_and_effects({
                    col_ray = col_ray,
                    no_decal = true,
                    no_sound = true
                })
            end
            self._camera_unit:base():play_anim_melee_item("hit_body")
        elseif self._on_melee_restart_drill and hit_unit:base() and (hit_unit:base().is_drill or hit_unit:base().is_saw) then
            hit_unit:base():on_melee_hit(managers.network:session():local_peer():id())
        else
            if bayonet_melee then
                self._unit:sound():play("knife_hit_gen", nil, false)
            else
                self:_play_melee_sound(melee_entry, "hit_gen", self._melee_attack_var)
            end
            self._camera_unit:base():play_anim_melee_item("hit_gen")
            managers.game_play_central:play_impact_sound_and_effects({
                col_ray = col_ray,
                effect = Idstring("effects/payday2/particles/impacts/fallback_impact_pd2"),
                no_decal = true,
                no_sound = true
            })
        end
        managers.rumble:play("melee_hit")
        managers.game_play_central:physics_push(col_ray)
        local character_unit, shield_knock
        local can_shield_knock = managers.player:has_category_upgrade("player", "shield_knock")
        if can_shield_knock and hit_unit:in_slot(8) and alive(hit_unit:parent()) and not hit_unit:parent():character_damage():is_immune_to_shield_knockback() then
            shield_knock = true
            character_unit = hit_unit:parent()
        end
        character_unit = character_unit or hit_unit
        if character_unit:character_damage() and character_unit:character_damage().damage_melee then
            local dmg_multiplier = 1
            if not managers.enemy:is_civilian(character_unit) and not managers.groupai:state():is_enemy_special(character_unit) then
                dmg_multiplier = dmg_multiplier * managers.player:upgrade_value("player", "non_special_melee_multiplier", 1)
            else
                dmg_multiplier = dmg_multiplier * managers.player:upgrade_value("player", "melee_damage_multiplier", 1)
            end
            dmg_multiplier = dmg_multiplier * managers.player:upgrade_value("player", "melee_" .. tostring(tweak_data.blackmarket.melee_weapons[melee_entry].stats.weapon_type) .. "_damage_multiplier", 1)
            if managers.player:has_category_upgrade("melee", "stacking_hit_damage_multiplier") then
                self._state_data.stacking_dmg_mul = self._state_data.stacking_dmg_mul or {}
                self._state_data.stacking_dmg_mul.melee = self._state_data.stacking_dmg_mul.melee or {nil, 0}
                local stack = self._state_data.stacking_dmg_mul.melee
                if stack[1] and t < stack[1] then
                    dmg_multiplier = dmg_multiplier * (1 + managers.player:upgrade_value("melee", "stacking_hit_damage_multiplier", 0) * stack[2])
                else
                    stack[2] = 0
                end
            end
            local health_ratio = self._ext_damage:health_ratio()
            local damage_health_ratio = managers.player:get_damage_health_ratio(health_ratio, "melee")
            if damage_health_ratio > 0 then
                local damage_ratio = damage_health_ratio
                dmg_multiplier = dmg_multiplier * (1 + managers.player:upgrade_value("player", "melee_damage_health_ratio_multiplier", 0) * damage_ratio)
            end
            dmg_multiplier = dmg_multiplier * managers.player:temporary_upgrade_value("temporary", "berserker_damage_multiplier", 1)
            local special_weapon = tweak_data.blackmarket.melee_weapons[melee_entry].special_weapon
            local action_data = {}
            action_data.variant = "melee"
            if special_weapon == "taser" then
                action_data.variant = "taser_tased"
                log('tased')
            end
            action_data.damage = shield_knock and 0 or damage * dmg_multiplier
            action_data.damage_effect = damage_effect
            log('damage_effect:'..tostring(damage_effect))
            action_data.attacker_unit = self._unit
            log('attacker_unit:'..tostring(attacker_unit))
            action_data.col_ray = col_ray
            if shield_knock then
                action_data.shield_knock = can_shield_knock
            end
            action_data.name_id = melee_entry
            log('name_id:'..tostring(name_id))
            action_data.charge_lerp_value = charge_lerp_value
            log('charge_lerp_value:'..tostring(charge_lerp_value))
            if managers.player:has_category_upgrade("melee", "stacking_hit_damage_multiplier") then
                self._state_data.stacking_dmg_mul = self._state_data.stacking_dmg_mul or {}
                self._state_data.stacking_dmg_mul.melee = self._state_data.stacking_dmg_mul.melee or {nil, 0}
                local stack = self._state_data.stacking_dmg_mul.melee
                if character_unit:character_damage().dead and not character_unit:character_damage():dead() then
                    stack[1] = t + managers.player:upgrade_value("melee", "stacking_hit_expire_t", 1)
                    stack[2] = math.min(stack[2] + 1, tweak_data.upgrades.max_melee_weapon_dmg_mul_stacks or 5)
                else
                    stack[1] = nil
                    stack[2] = 0
                end
            end
            local defense_data = character_unit:character_damage():damage_melee(action_data)
            self:_check_melee_dot_damage(col_ray, defense_data, melee_entry)
            self:_perform_sync_melee_damage(hit_unit, col_ray, action_data.damage)
            return defense_data
        else
            self:_perform_sync_melee_damage(hit_unit, col_ray, damage)
        end
    else
    end
    if managers.player:has_category_upgrade("melee", "stacking_hit_damage_multiplier") then
        self._state_data.stacking_dmg_mul = self._state_data.stacking_dmg_mul or {}
        self._state_data.stacking_dmg_mul.melee = self._state_data.stacking_dmg_mul.melee or {nil, 0}
        local stack = self._state_data.stacking_dmg_mul.melee
        stack[1] = nil
        stack[2] = 0
    end
    return col_ray
end